#!/usr/bin/env python
# -*- coding: utf-8 -*-

__title__    = "zabbixsend"
__author__   = "Anton Batenev"
__license__  = "BSD"

import sys

# PEP-3108
try:
    from urllib.parse   import urlencode as zabbix_urlencode
    from urllib.request import urlopen   as zabbix_urlopen
except ImportError:
    from urllib         import urlencode as zabbix_urlencode
    from urllib2        import urlopen   as zabbix_urlopen

HOST = "127.0.0.1"
PORT = "32445"

if __name__ == "__main__":
    if len(sys.argv) != 4:
        print("Usage: {0} <address> <subject> <message>".format(sys.argv[0]))
        sys.exit(1)

    address = str(sys.argv[1])
    subject = str(sys.argv[2])
    message = str(sys.argv[3])

    args = {
        "address" : address,
        "message" : message
    }

    try:
        url = "http://{0}:{1}/send?{2}".format(HOST, PORT, zabbix_urlencode(args))
        result = zabbix_urlopen(url)
    except Exception as e:
        sys.exit(1)

    sys.exit(0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__title__    = "avbot"
__author__   = "Anton Batenev"
__license__  = "BSD"

import cv2
import pyaudio
import threading

from echobot import *
from pytoxcore import ToxAV, ToxAVException


from pkg_resources import parse_version

if parse_version(cv2.__version__) >= parse_version("3"):
    CV_CAP_PROP_FPS          = cv2.CAP_PROP_FPS
    CV_CAP_PROP_FRAME_WIDTH  = cv2.CV_CAP_PROP_FRAME_WIDTH
    CV_CAP_PROP_FRAME_HEIGHT = cv2.CV_CAP_PROP_FRAME_HEIGHT
else:
    CV_CAP_PROP_FPS          = cv2.cv.CV_CAP_PROP_FPS
    CV_CAP_PROP_FRAME_WIDTH  = cv2.cv.CV_CAP_PROP_FRAME_WIDTH
    CV_CAP_PROP_FRAME_HEIGHT = cv2.cv.CV_CAP_PROP_FRAME_HEIGHT


class AVCall(object):
    """
    Описатель звонка
    """
    def __init__(self, friend_number, video_enabled, audio_enabled):
        """
        Аргументы:
            friend_number (int)  -- Номер друга
            audio_enabled (bool) -- Включена передача аудио
            video_enabled (bool) -- Включена передача видео
        """
        self.friend_number = friend_number
        self.video_enabled = video_enabled
        self.audio_enabled = audio_enabled


class AVBot(ToxAV):
    """
    AV Бот
    """
    def __init__(self, core):
        """
        Аргументы:
            core (ToxCore) -- Экземпляр ToxCore
        """
        self.core    = core
        self.running = True

        self.calls = {}

        self.audio         = None
        self.audio_stream  = None
        self.audio_thread  = None
        self.audio_running = False

        self.video         = None
        self.video_thread  = None
        self.video_running = False

        super(AVBot, self).__init__(self.core)

        self.toxav_video_frame_format_set(self.TOXAV_VIDEO_FRAME_FORMAT_BGR)

        self.iterate_thread = threading.Thread(target = self.iterate_cb)
        self.iterate_thread.start()


    def stop(self):
        """
        Остановка всех потоков
        """
        self.core.verbose("stopping...")

        self.running = False

        self.stop_video_thread()
        self.stop_audio_thread()

        self.iterate_thread.join()

        self.toxav_kill()

        self.core.verbose("stopped")


    def iterate_cb(self):
        """
        Рабочий цикл, запускается в отдельном потоке относительно ToxCore
        """
        while self.running:
            self.toxav_iterate()
            interval = self.toxav_iteration_interval()
            time.sleep(float(interval) / 1000.0)


    def start_video_thread(self):
        """
        Запуск потока кодирования видео
        """
        if self.video_thread != None:
            return

        self.video_running = True

        self.video = cv2.VideoCapture(0)
        self.video.set(CV_CAP_PROP_FPS, 25)
        self.video.set(CV_CAP_PROP_FRAME_WIDTH, 640)
        self.video.set(CV_CAP_PROP_FRAME_HEIGHT, 480)

        self.video_thread = threading.Thread(target = self.video_cb)
        self.video_thread.start()


    def stop_video_thread(self):
        """
        Остановка потока кодирования видео
        """
        if self.video_thread == None:
            return

        self.video_running = False

        self.video_thread.join()

        self.video_thread = None
        self.video        = None


    def video_cb(self):
        """
        Рабочий цикл отправки видео
        """
        while self.video_running:
            try:
                result, frame = self.video.read()
                if result:
                    height, width, channels = frame.shape
                    for call in itervalues(self.calls):
                        if call.video_enabled:
                            try:
                                self.toxav_video_send_bgr_frame(call.friend_number, width, height, frame.tostring())
                            except ToxAVException as e:
                                self.core.verbose("ToxAVException: {0}".format(e))
            except Exception as e:
                self.core.verbose("{0}".format(e))

            time.sleep(0.01)


    def start_audio_thread(self):
        """
        Запуск потока кодирования аудио
        """
        if self.audio_thread != None:
            return

        self.audio_running = True

        self.audio_device       = "USB 2.0 Camera: USB Audio (hw:1,0)"
        self.audio_device_index = None
        self.audio_rate         = 8000
        self.audio_channels     = 1
        self.audio_duration     = 60
        self.audio_sample_count = self.audio_rate * self.audio_channels * self.audio_duration / 1000

        self.audio = pyaudio.PyAudio()

        for i in range(0, self.audio.get_device_count() - 1):
            name = self.audio.get_device_info_by_index(i)["name"]
            self.core.debug("Found device {0}: {1}".format(i, name))
            if name == self.audio_device:
                self.audio_device_index = i

        if self.audio_device_index != None:
            device_info = self.audio.get_device_info_by_index(self.audio_device_index)
            self.core.debug("Use found audio device: {0}".format(device_info["name"]))
        else:
            device_info = self.audio.get_default_input_device_info()
            self.core.debug("Use default audio device: {0}".format(device_info["name"]))
            self.audio_device_index = device_info["index"]

        format_supported = self.audio.is_format_supported(self.audio_rate, self.audio_device_index, self.audio_channels, pyaudio.paInt16)
        self.core.debug("Input audio format supported: {0}".format("yes" if format_supported else "no"))

        if not format_supported:
            self.audio_running = False
            self.self.audio    = None
            return

        self.audio_stream = self.audio.open(format = pyaudio.paInt16,
                                            rate = self.audio_rate,
                                            channels = self.audio_channels,
                                            input = True,
                                            input_device_index = self.audio_device_index,
                                            frames_per_buffer = self.audio_sample_count * 10)

        self.audio_thread = threading.Thread(target = self.audio_cb)
        self.audio_thread.start()


    def stop_audio_thread(self):
        """
        Остановка потока кодирования аудио
        """
        if self.audio_thread == None:
            return

        self.audio_running = False

        self.audio_thread.join()

        self.audio_thread = None
        self.audio_stream = None
        self.audio        = None


    def audio_cb(self):
        """
        Рабочий цикл отправки видео
        """
        while self.audio_running:
            try:
                pcm = self.audio_stream.read(self.audio_sample_count)
                if pcm:
                    for call in itervalues(self.calls):
                        if call.audio_enabled:
                            try:
                                self.toxav_audio_send_frame(call.friend_number, pcm, self.audio_sample_count, self.audio_channels, self.audio_rate)
                            except ToxAVException as e:
                                self.core.verbose("ToxAVException: {0}".format(e))
            except Exception as e:
                self.core.verbose("{0}".format(e))

            time.sleep(0.01)


    def toxav_call_cb(self, friend_number, audio_enabled, video_enabled):
        """
        Событие входящего звонка
        (см. toxav_call_cb)

        Аргументы:
            friend_number (int)  -- Номер друга
            audio_enabled (bool) -- Включена передача аудио
            video_enabled (bool) -- Включена передача видео
        """
        if self.running:
            friend_name = self.core.tox_friend_get_name(friend_number)
            self.core.verbose("Friend {0}/{1} call with audio = {2} and video = {3}".format(friend_name, friend_number, audio_enabled, video_enabled))

            self.calls[friend_number] = AVCall(friend_number, audio_enabled, video_enabled)

            self.toxav_answer(friend_number, 32, 5000)

            self.start_audio_thread()
            self.start_video_thread()


    def toxav_call_state_cb(self, friend_number, state):
        """
        Событие смены состояния звонка
        (см. toxav_call_state_cb)

        Аргументы:
            friend_number (int) -- Номер друга
            state         (int) -- Битовая маска состояний
        """
        if self.running:
            friend_name = self.core.tox_friend_get_name(friend_number)
            self.core.verbose("Friend {0}/{1} change call state = {2}".format(friend_name, friend_number, state))

            if state == self.TOXAV_FRIEND_CALL_STATE_FINISHED or state == self.TOXAV_FRIEND_CALL_STATE_ERROR:
                del self.calls[friend_number]
                if len(self.calls) == 0:
                    self.stop_video_thread()
                    self.stop_audio_thread()

            if state & self.TOXAV_FRIEND_CALL_STATE_ACCEPTING_A:
                self.calls[friend_number].audio_enabled = True

            if state & self.TOXAV_FRIEND_CALL_STATE_ACCEPTING_V:
                self.calls[friend_number].video_enabled = True


    def toxav_bit_rate_status_cb(self, friend_number, audio_bit_rate, video_bit_rate):
        """
        Событие перегрузки сети
        (см. toxav_bit_rate_status_cb)

        Аргументы:
            friend_number  (int) -- Номер друга
            audio_bit_rate (int) -- Рекомендуемая скорость аудио
            video_bit_rate (int) -- Рекомендуемая скорость видео
        """
        if self.running:
            friend_name = self.core.tox_friend_get_name(friend_number)
            self.core.verbose("Friend {0}/{1} change audio bitrate = {2} and video bitrate = {3}".format(friend_name, friend_number, audio_bit_rate, video_bit_rate))


if __name__ == "__main__":
    regexp  = re.compile("--config=(.*)")
    cfgfile = [match.group(1) for arg in sys.argv for match in [regexp.search(arg)] if match]

    if len(cfgfile) == 0:
        cfgfile = "echobot.cfg"
    else:
        cfgfile = cfgfile[0]

    options = EchoBotOptions(EchoBotOptions.loadOptions(cfgfile))

    bot   = EchoBot(options)
    botav = AVBot(bot)

    try:
        bot.run()
    except KeyboardInterrupt:
        pass

    botav.stop()
    bot.save_file()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__title__    = "echoavbot"
__author__   = "Anton Batenev"
__license__  = "BSD"

import threading

from echobot import *
from pytoxcore import ToxAV, ToxAVException


class EchoAVBot(ToxAV):
    """
    AV Бот
    """
    def __init__(self, core):
        """
        Аргументы:
            core (ToxCore) -- Экземпляр ToxCore
        """
        self.core    = core
        self.running = True

        super(EchoAVBot, self).__init__(self.core)

        self.toxav_video_frame_format_set(self.TOXAV_VIDEO_FRAME_FORMAT_BGR)

        self.iterate_thread = threading.Thread(target = self.iterate_cb)
        self.iterate_thread.start()


    def stop(self):
        """
        Остановка всех потоков
        """
        self.core.verbose("stopping...")
        self.running = False
        self.iterate_thread.join()
        self.toxav_kill()
        self.core.verbose("stopped")


    def iterate_cb(self):
        """
        Рабочий цикл, запускается в отдельном потоке относительно ToxCore
        """
        while self.running:
            self.toxav_iterate()
            interval = self.toxav_iteration_interval()
            time.sleep(float(interval) / 1000.0)


    def toxav_call_cb(self, friend_number, audio_enabled, video_enabled):
        """
        Событие входящего звонка
        (см. toxav_call_cb)

        Аргументы:
            friend_number (int)  -- Номер друга
            audio_enabled (bool) -- Включена передача аудио
            video_enabled (bool) -- Включена передача видео
        """
        if self.running:
            friend_name = self.core.tox_friend_get_name(friend_number)
            self.core.verbose("Friend {0}/{1} call with audio = {2} and video = {3}".format(friend_name, friend_number, audio_enabled, video_enabled))
            self.toxav_answer(friend_number, 32, 5000)


    def toxav_call_state_cb(self, friend_number, state):
        """
        Событие смены состояния звонка
        (см. toxav_call_state_cb)

        Аргументы:
            friend_number (int) -- Номер друга
            state         (int) -- Битовая маска состояний
        """
        if self.running:
            friend_name = self.core.tox_friend_get_name(friend_number)
            self.core.verbose("Friend {0}/{1} change call state = {2}".format(friend_name, friend_number, state))


    def toxav_bit_rate_status_cb(self, friend_number, audio_bit_rate, video_bit_rate):
        """
        Событие перегрузки сети
        (см. toxav_bit_rate_status_cb)

        Аргументы:
            friend_number  (int) -- Номер друга
            audio_bit_rate (int) -- Рекомендуемая скорость аудио
            video_bit_rate (int) -- Рекомендуемая скорость видео
        """
        if self.running:
            friend_name = self.core.tox_friend_get_name(friend_number)
            self.core.verbose("Friend {0}/{1} change audio bitrate = {2} and video bitrate = {3}".format(friend_name, friend_number, audio_bit_rate, video_bit_rate))


    def toxav_audio_receive_frame_cb(self, friend_number, pcm, sample_count, channels, sampling_rate):
        """
        Событие получения аудио-кадра
        (см. toxav_audio_receive_frame_cb)

        Аргументы:
            friend_number (int) -- Номер друга
            pcm           (str) -- Данные PCM
            sample_count  (int) -- Количество сэмплов
            channels      (int) -- Количество каналов
            sampling_rate (int) -- Частота дискретизации
        """
        if self.running:
            try:
                self.toxav_audio_send_frame(friend_number, pcm, sample_count, channels, sampling_rate)
            except ToxAVException as e:
                self.core.verbose("ToxAVException: {0}".format(e))


    def toxav_video_receive_frame_cb(self, friend_number, width, height, bgr):
        """
        Событие получения видео-кадра
        (см. toxav_video_receive_frame_cb)

        Аргументы:
            friend_number (int) -- Номер друга
            width         (int) -- Ширина кадра
            height        (int) -- Высота кадра
            bgr           (str) -- Данные в формате BGR (см. toxav_video_frame_format_set)
        """
        if self.running:
            try:
                self.toxav_video_send_bgr_frame(friend_number, width, height, bgr)
            except ToxAVException as e:
                self.core.verbose("ToxAVException: {0}".format(e))


if __name__ == "__main__":
    regexp  = re.compile("--config=(.*)")
    cfgfile = [match.group(1) for arg in sys.argv for match in [regexp.search(arg)] if match]

    if len(cfgfile) == 0:
        cfgfile = "echobot.cfg"
    else:
        cfgfile = cfgfile[0]

    options = EchoBotOptions(EchoBotOptions.loadOptions(cfgfile))

    bot   = EchoBot(options)
    botav = EchoAVBot(bot)

    try:
        bot.run()
    except KeyboardInterrupt:
        pass

    botav.stop()
    bot.save_file()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__title__    = "toxdns"
__author__   = "Anton Batenev"
__license__  = "BSD"

import dns.resolver

from pytoxcore import ToxDNS, ToxDNSException


class ToxDNSResolver(object):
    """
    Ресолвер адресов Tox
    """
    def __init__(self, v1_fallback = True, cache_keys = True):
        """
        Аргументы:
            v1_fallback (bool) -- Разрешить fallback на tox1
            cache_keys  (bool) -- Кэшировать публичные ключи
        """
        self.v1_fallback = v1_fallback
        self.cache_keys  = cache_keys

        self.cached_keys = {}


    def resolve_txt_record(self, query):
        """
        Получение TXT записи из DNS

        Аргументы:
            query (str) -- Запрос (имя домена)

        Результат (str)
            Значение TXT записи
        """
        response = dns.resolver.query(query, "TXT")
        if response:
            for record in response:
                return str(record).replace('"', "")

        raise ValueError("TXT record not found for {0}".format(query))


    def parse_txt_record(self, record):
        """
        Разбор TXT записи

        Аргументы:
            record (str) -- Значение TXT записи

        Результат (dict)
            Словарь ключей и значений в записи
        """
        result = {}

        comma_parts = record.split(";")
        for comma_part in comma_parts:
            comma_part = comma_part.strip()
            if len(comma_part) == 0:
                continue

            keyval = comma_part.split("=", 1)
            if len(keyval) != 2:
                continue

            key   = keyval[0].strip()
            value = keyval[1].strip()

            result[key] = value

        return result


    def is_v_record(self, record, version):
        """
        Проверка записи на соответствие версии tox

        Аргументы:
            record  (dict) -- Значение записи
            version (int)  -- Требуемая версия

        Результат (bool)
            True, если запись требуемой версии
        """
        if not ("id" in record and "v" in record):
            return False
        if record["v"].lower() != "tox{0}".format(version):
            return False
        return True


    def is_v3_record(self, record):
        """
        Проверка записи на соответствие версии tox3

        Аргументы:
            record (dict) -- Значение записи

        Результат (bool)
            True, если запись версии tox3
        """
        return self.is_v_record(record, 3)


    def is_v1_record(self, record):
        """
        Проверка записи на соответствие версии tox1

        Аргументы:
            record (dict) -- Значение записи

        Результат (bool)
            True, если запись версии tox1
        """
        return self.is_v_record(record, 1)


    def user_domain(self, request):
        """
        Получение части имени пользователя и домена из запроса

        Аргументы:
            request (str) -- Запрос

        Результат (tuple): user (str), domain (str)
            Имя пользователя и домен
        """
        request_parts = request.split("@", 1)
        if len(request_parts) != 2:
            raise ValueError("Request must be user@domain form.")

        return (request_parts[0].lower(), request_parts[1].lower())


    def public_key(self, domain):
        """
        Получение публичного ключа для домена

        Аргументы:
            domain (str) -- Домен

        Результат (str):
            Публичный ключ
        """
        if self.cache_keys and domain in self.cached_keys:
            return self.cached_keys[domain]["key"]

        request = "_tox.{0}".format(domain)
        record  = self.resolve_txt_record(request)

        if len(record) != ToxDNS.TOX_CLIENT_ID_SIZE * 2:
            raise ValueError("TXT record length for {0} is {1}, but {2} required".format(request, len(record), ToxDNS.TOX_CLIENT_ID_SIZE * 2))

        if self.cache_keys:
            self.cached_keys[domain] = { "key": record, "dns": ToxDNS(record) }

        return record


    def tox_dns(self, domain, public_key):
        """
        Получение экземпляра ToxDNS (из кэша или создание нового)

        Аргументы:
            domain     (str) -- Домен
            public_key (str) -- Публичный ключ

        Результат (ToxDNS):
            Экземпляр ToxDNS
        """
        if self.cache_keys and domain in self.cached_keys:
            return self.cached_keys[domain]["dns"]
        return ToxDNS(public_key)


    def tox3_request(self, request):
        """
        Выполнение tox3 запроса

        Аргументы:
            request (str) -- Запрос (user@domain)

        Результат (str):
            ToxID пользователя
        """
        user, domain = self.user_domain(request)

        public_key = self.public_key(domain)
        tox_dns    = self.tox_dns(domain, public_key)

        record_id, request_id = tox_dns.tox_generate_dns3_string(user)

        request = "_{0}._tox.{1}".format(record_id, domain)
        record  = self.resolve_txt_record(request)
        record  = self.parse_txt_record(record)

        if self.is_v3_record(record) == False:
            raise ValueError("TXT record for {0} is not {1} record".format(request, "tox3"))

        return tox_dns.tox_decrypt_dns3_TXT(record["id"], request_id)


    def tox1_request(self, request):
        """
        Выполнение tox1 запроса

        Аргументы:
            request (str) -- Запрос (user@domain)

        Результат (str):
            ToxID пользователя
        """
        user, domain = self.user_domain(request)

        request = "{0}._tox.{1}".format(user, domain)
        record  = self.resolve_txt_record(request)
        record  = self.parse_txt_record(record)

        if self.is_v1_record(record) == False:
            raise ValueError("TXT record for {0} is not {1} record".format(request, "tox1"))

        if "sign" in record:
            # TODO: sign == crypto_sign_ed25519(user + tox_id)
            pass

        return record["id"]


    def tox_request(self, request):
        """
        Выполнение tox-dns запроса

        Аргументы:
            request (str) -- Запрос (user@domain)

        Результат (str):
            ToxID пользователя
        """
        try:
            return self.tox3_request(request)
        except ValueError as e:
            if not self.v1_fallback:
                raise e
            return self.tox1_request(request)


if __name__ == "__main__":
    resolver = ToxDNSResolver()

    # к сожланию единственный работающий ToxDNS (toxme.io) был убит владельцем
    # (см. https://github.com/Antox/Antox/issues/193)
    #print(resolver.tox1_request("tox@toxme.io"))
    #print(resolver.tox3_request("tox@toxme.io"))
    #print(resolver.tox_request("tox@toxme.io"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__title__    = "zabbixbot"
__author__   = "Anton Batenev"
__license__  = "BSD"

import sys
import os
import time
import re
import flask
import threading

from pytoxcore import ToxCore, ToxCoreException


# PEP-8
try:
    import configparser as ZabbixBotConfigParser
except ImportError:
    import ConfigParser as ZabbixBotConfigParser


app = flask.Flask(__name__)


class ZabbixBotOptions(object):
    """
    Опции приложения
    """
    def __init__(self, options):
        """
        Аргументы:
            options (dict) -- опции приложения
        """
        self.debug          = self._bool(options["debug"])
        self.verbose        = self._bool(options["verbose"]) or self.debug
        self.bind           = str(options["bind"])
        self.port           = int(options["port"])
        self.name           = str(options["name"])
        self.status_message = str(options["status_message"])
        self.save_file      = str(options["save_file"])
        self.save_tmp_file  = str(options["save_tmp_file"])
        self.save_interval  = int(options["save_interval"])
        self.bootstrap_host = str(options["bootstrap_host"])
        self.bootstrap_port = int(options["bootstrap_port"])
        self.bootstrap_key  = str(options["bootstrap_key"])
        self.ipv6_enabled   = self._bool(options["ipv6_enabled"])
        self.udp_enabled    = self._bool(options["udp_enabled"])

        proxy_type = options["proxy_type"].lower()
        if len(proxy_type) == 0:
            self.proxy_type = ToxCore.TOX_PROXY_TYPE_NONE
        elif proxy_type == "http":
            self.proxy_type = ToxCore.TOX_PROXY_TYPE_HTTP
        elif proxy_type == "socks":
            self.proxy_type = ToxCore.TOX_PROXY_TYPE_SOCKS5
        else:
            raise ValueError("Unknown proxy type: {0}".format(options["proxy_type"]))

        self.proxy_host = str(options["proxy_host"])
        self.proxy_port = int(options["proxy_port"])
        self.start_port = int(options["start_port"])
        self.end_port   = int(options["end_port"])
        self.tcp_port   = int(options["tcp_port"])


    def __repr__(self):
        return "{0!s}({1!r})".format(self.__class__, self.__dict__)


    @staticmethod
    def _bool(value):
        """
        Преобразование строкового значения к булевому

        Аргументы:
            value (str|bool) -- Строковое представление булева значения

        Результат (bool):
            Результат преобразования строкового значения к булеву - [true|yes|t|y|1] => True, иначе False
        """
        if type(value) is bool:
            return value

        value = value.lower().strip()

        if value == "true" or value == "yes" or value == "t" or value == "y" or value == "1":
            return True

        return False


    @staticmethod
    def defaultOptions():
        """
        Опции по умолчанию

        Результат (dict):
            Словарь опций по умолчанию
        """
        tox_opts = ToxCore.tox_options_default()

        options = {
            "debug"           : "no",
            "verbose"         : "yes",
            "bind"            : "127.0.0.1",
            "port"            : "32445",
            "name"            : "ZabbixBot",
            "status_message"  : "Think Stability",
            "save_file"       : "zabbixbot.data",
            "save_tmp_file"   : "zabbixbot.data.tmp",
            "save_interval"   : "300",
            "bootstrap_host"  : "178.62.250.138",   # https://wiki.tox.chat/users/nodes
            "bootstrap_port"  : "32445",
            "bootstrap_key"   : "788236D34978D1D5BD822F0A5BEBD2C53C64CC31CD3149350EE27D4D9A2F9B6B",
            "ipv6_enabled"    : "yes" if tox_opts["ipv6_enabled"] else "no",
            "udp_enabled"     : "yes" if tox_opts["udp_enabled"]  else "no",
            "proxy_type"      : "",
            "proxy_host"      : "" if tox_opts["proxy_host"] is None else tox_opts["proxy_host"],
            "proxy_port"      : str(tox_opts["proxy_port"]),
            "start_port"      : str(tox_opts["start_port"]),
            "end_port"        : str(tox_opts["end_port"]),
            "tcp_port"        : str(tox_opts["tcp_port"])
        }

        if tox_opts["proxy_type"] == ToxCore.TOX_PROXY_TYPE_SOCKS5:
            options["proxy_type"] = "socks"
        elif tox_opts["proxy_type"] == ToxCore.TOX_PROXY_TYPE_HTTP:
            options["proxy_type"] = "http"
        elif tox_opts["proxy_type"] != ToxCore.TOX_PROXY_TYPE_NONE:
            raise NotImplementedError("Unknown proxy_type: {0}".format(tox_opts["proxy_type"]))

        return options


    @staticmethod
    def loadOptions(filename, options = None):
        """
        Чтение секции echobot INI файла zabbixbot.cfg

        Аргументы:
            filename (str)  -- Имя INI файла
            options  (dict) -- Базовая конфигурация

        Результат (dict):
            Конфигурация приложения на основе файла конфигурации
        """
        if options is None:
            options = ZabbixBotOptions.defaultOptions()

        options = options.copy()

        parser = ZabbixBotConfigParser.ConfigParser()

        parser.read(filename)

        for section in parser.sections():
            name = section.lower()
            if name == "zabbixbot":
                for option in parser.options(section):
                    options[option.lower()] = parser.get(section, option).strip()

        return options


class ZabbixBot(ToxCore):
    """
    Бот
    """
    def __init__(self, options):
        """
        Аргументы:
            options (ZabbixBotOptions) -- Опции приложения
        """
        self.options = options

        tox_opts = {
            "ipv6_enabled" : self.options.ipv6_enabled,
            "udp_enabled"  : self.options.udp_enabled,
            "proxy_type"   : self.options.proxy_type,
            "proxy_host"   : self.options.proxy_host,
            "proxy_port"   : self.options.proxy_port,
            "start_port"   : self.options.start_port,
            "end_port"     : self.options.end_port,
            "tcp_port"     : self.options.tcp_port
        }

        if os.path.isfile(self.options.save_file):
            self.debug("Load data from file: {0}".format(self.options.save_file))
            with open(self.options.save_file, "rb") as f:
                tox_opts["savedata_type"] = ToxCore.TOX_SAVEDATA_TYPE_TOX_SAVE
                tox_opts["savedata_data"] = f.read()

        super(ZabbixBot, self).__init__(tox_opts)

        self.debug("Set self name: {0}".format(self.options.name))
        self.tox_self_set_name(self.options.name)

        self.debug("Set self status: {0}".format(self.options.status_message))
        self.tox_self_set_status_message(self.options.status_message)

        self.debug("Get self ToxID: {0}".format(self.tox_self_get_address()))

        self.running = True

        self.thread = threading.Thread(target = self.run)
        self.thread.start()


    def debug(self, message):
        """
        Вывод отладочной информации

        Аргументы:
            message (str) -- Сообщение для вывода
        """
        if self.options.debug:
            sys.stderr.write("[{0}] {1}\n".format(time.strftime("%Y-%m-%d %H:%M:%S"), message))


    def verbose(self, message):
        """
        Вывод расширенной информации

        Аргументы:
            message (str) -- Сообщение для вывода
        """
        if self.options.verbose:
            sys.stderr.write("[{0}] {1}\n".format(time.strftime("%Y-%m-%d %H:%M:%S"), message))


    def stop(self):
        """
        Остановка всех потоков
        """
        self.running = False
        self.thread.join()


    def run(self):
        self.debug("Connecting to: {0} {1} {2}".format(self.options.bootstrap_host, self.options.bootstrap_port, self.options.bootstrap_key))
        self.tox_bootstrap(self.options.bootstrap_host, self.options.bootstrap_port, self.options.bootstrap_key)
        self.debug("Connected to: {0} {1} {2}".format(self.options.bootstrap_host, self.options.bootstrap_port, self.options.bootstrap_key))

        checked       = False
        savetime      = 0
        save_interval = self.options.save_interval * 1000

        while self.running:
            status = self.tox_self_get_connection_status()

            if not checked and status != ToxCore.TOX_CONNECTION_NONE:
                checked = True
            if checked and status == ToxCore.TOX_CONNECTION_NONE:
                self.debug("Connecting to: {0} {1} {2}".format(self.options.bootstrap_host, self.options.bootstrap_port, self.options.bootstrap_key))
                self.tox_bootstrap(self.options.bootstrap_host, self.options.bootstrap_port, self.options.bootstrap_key)
                self.debug("Connected to: {0} {1} {2}".format(self.options.bootstrap_host, self.options.bootstrap_port, self.options.bootstrap_key))
                checked = False

            self.tox_iterate()

            interval = self.tox_iteration_interval()

            time.sleep(float(interval) / 1000.0)

            savetime += interval
            if savetime > save_interval:
                self.save_file()
                savetime = 0


    def save_file(self):
        """
        Сохранение данных
        """
        self.debug("Save data to file: {0}".format(self.options.save_tmp_file))

        with open(self.options.save_tmp_file, "wb") as f:
            f.write(self.tox_get_savedata())

        self.debug("Move data to file: {0}".format(self.options.save_file))
        os.rename(self.options.save_tmp_file, self.options.save_file)


def rest_success(result):
    return "{0}\n".format(result)


def rest_error(result, code = 500):
    return ("{0}\n".format(result), code)


@app.route("/status", methods=["GET"])
def rest_status():
    try:
        status = bot.tox_self_get_connection_status()
        if status == ToxCore.TOX_CONNECTION_NONE:
            result = "NONE"
        elif status == ToxCore.TOX_CONNECTION_TCP or status == ToxCore.TOX_CONNECTION_UDP:
            result = "ONLINE"
        else:
            result = "UNKNOWN"

        return rest_success(result)
    except ToxCoreException as e:
        return rest_error(e.message)


@app.route("/address", methods=["GET"])
def rest_address():
    try:
        result = bot.tox_self_get_address()
        return rest_success(result)
    except ToxCoreException as e:
        return rest_error(e.message)


@app.route("/send", methods=["GET"])
def rest_message():
    try:
        address = flask.request.args.get("address", "")
        message = flask.request.args.get("message", "")

        public_key = address[0:ToxCore.TOX_PUBLIC_KEY_SIZE * 2]

        try:
            friend_number = bot.tox_friend_by_public_key(public_key)
        except ToxCoreException as e:
            result = bot.tox_friend_add(address, message)
            return rest_success(result)

        result = bot.tox_friend_send_message(friend_number, ToxCore.TOX_MESSAGE_TYPE_NORMAL, message)
        return rest_success(result)
    except ToxCoreException as e:
        return rest_error(e.message)


if __name__ == "__main__":
    regexp  = re.compile("--config=(.*)")
    cfgfile = [match.group(1) for arg in sys.argv for match in [regexp.search(arg)] if match]

    if len(cfgfile) == 0:
        cfgfile = "zabbixbot.cfg"
    else:
        cfgfile = cfgfile[0]

    options = ZabbixBotOptions(ZabbixBotOptions.loadOptions(cfgfile))

    bot = ZabbixBot(options)

    app.run(host = options.bind, port = options.port, debug = options.debug, use_reloader = False)

    bot.stop()



from distutils.core import setup, Extension

setup(
    name         = "pytoxcore",
    version      = "0.2.3",
    description  = 'Python binding for ToxCore',
    author       = 'Anton Batenev',
    author_email = 'antonbatenev@yandex.ru',
    url          = 'http://github.com/abbat/pytoxcore',
    license      = 'GPL',
    ext_modules  = [
        Extension(
            "pytoxcore",
            sources            = ["pytox.c", "pytoxcore.c", "pytoxav.c", "pytoxdns.c"],
            define_macros      = [],
            include_dirs       = ["/usr/tox/include"],
            library_dirs       = ["/usr/tox/lib"],
            extra_compile_args = ["-Wall", "-Werror", "-Wno-declaration-after-statement"],
            libraries          = ["toxcore", "toxav", "toxdns", "sodium", "vpx", "opus", "rt"]
        )
    ]
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

__title__    = "echobot"
__author__   = "Anton Batenev"
__license__  = "BSD"

import sys
import os
import time
import re

from pytoxcore import ToxCore, ToxCoreException


# PEP-8
try:
    import configparser as EchoBotConfigParser
except ImportError:
    import ConfigParser as EchoBotConfigParser


# PEP-469
try:
    dict.iteritems
except AttributeError:
    def itervalues(d):
        return iter(d.values())
    def iteritems(d):
        return iter(d.items())
    def listvalues(d):
        return list(d.values())
    def listitems(d):
        return list(d.items())
else:
    def itervalues(d):
        return d.itervalues()
    def iteritems(d):
        return d.iteritems()
    def listvalues(d):
        return d.values()
    def listitems(d):
        return d.items()


class EchoBotOptions(object):
    """
    Опции приложения
    """
    def __init__(self, options):
        """
        Аргументы:
            options (dict) -- опции приложения
        """
        self.debug          = self._bool(options["debug"])
        self.verbose        = self._bool(options["verbose"]) or self.debug
        self.name           = str(options["name"])
        self.status_message = str(options["status_message"])
        self.avatar         = str(options["avatar"])
        self.save_file      = str(options["save_file"])
        self.save_tmp_file  = str(options["save_tmp_file"])
        self.save_interval  = int(options["save_interval"])
        self.bootstrap_host = str(options["bootstrap_host"])
        self.bootstrap_port = int(options["bootstrap_port"])
        self.bootstrap_key  = str(options["bootstrap_key"])
        self.ipv6_enabled   = self._bool(options["ipv6_enabled"])
        self.udp_enabled    = self._bool(options["udp_enabled"])

        proxy_type = options["proxy_type"].lower()
        if len(proxy_type) == 0:
            self.proxy_type = ToxCore.TOX_PROXY_TYPE_NONE
        elif proxy_type == "http":
            self.proxy_type = ToxCore.TOX_PROXY_TYPE_HTTP
        elif proxy_type == "socks":
            self.proxy_type = ToxCore.TOX_PROXY_TYPE_SOCKS5
        else:
            raise ValueError("Unknown proxy type: {0}".format(options["proxy_type"]))

        self.proxy_host = str(options["proxy_host"])
        self.proxy_port = int(options["proxy_port"])
        self.start_port = int(options["start_port"])
        self.end_port   = int(options["end_port"])
        self.tcp_port   = int(options["tcp_port"])

        self.accept_avatars  = self._bool(options["accept_avatars"])
        self.max_avatar_size = int(options["max_avatar_size"])
        self.avatars_path    = str(options["avatars_path"])
        self.accept_files    = self._bool(options["accept_files"])
        self.max_file_size   = int(options["max_file_size"])
        self.files_path      = str(options["files_path"])


    def __repr__(self):
        return "{0!s}({1!r})".format(self.__class__, self.__dict__)


    @staticmethod
    def _bool(value):
        """
        Преобразование строкового значения к булевому

        Аргументы:
            value (str|bool) -- Строковое представление булева значения

        Результат (bool):
            Результат преобразования строкового значения к булеву - [true|yes|t|y|1] => True, иначе False
        """
        if type(value) is bool:
            return value

        value = value.lower().strip()

        if value == "true" or value == "yes" or value == "t" or value == "y" or value == "1":
            return True

        return False


    @staticmethod
    def defaultOptions():
        """
        Опции по умолчанию

        Результат (dict):
            Словарь опций по умолчанию
        """
        tox_opts = ToxCore.tox_options_default()

        options = {
            "debug"           : "yes",
            "verbose"         : "yes",
            "name"            : "EchoBot",
            "status_message"  : "Think Safety",
            "avatar"          : "echobot.png",
            "save_file"       : "echobot.data",
            "save_tmp_file"   : "echobot.data.tmp",
            "save_interval"   : "300",
            "bootstrap_host"  : "178.62.250.138",   # https://wiki.tox.chat/users/nodes
            "bootstrap_port"  : "33445",
            "bootstrap_key"   : "788236D34978D1D5BD822F0A5BEBD2C53C64CC31CD3149350EE27D4D9A2F9B6B",
            "ipv6_enabled"    : "yes" if tox_opts["ipv6_enabled"] else "no",
            "udp_enabled"     : "yes" if tox_opts["udp_enabled"]  else "no",
            "proxy_type"      : "",
            "proxy_host"      : "" if tox_opts["proxy_host"] is None else tox_opts["proxy_host"],
            "proxy_port"      : str(tox_opts["proxy_port"]),
            "start_port"      : str(tox_opts["start_port"]),
            "end_port"        : str(tox_opts["end_port"]),
            "tcp_port"        : str(tox_opts["tcp_port"]),
            "accept_avatars"  : "no",
            "max_avatar_size" : "0",
            "avatars_path"    : "",
            "accept_files"    : "no",
            "max_file_size"   : "0",
            "files_path"      : "",
        }

        if tox_opts["proxy_type"] == ToxCore.TOX_PROXY_TYPE_SOCKS5:
            options["proxy_type"] = "socks"
        elif tox_opts["proxy_type"] == ToxCore.TOX_PROXY_TYPE_HTTP:
            options["proxy_type"] = "http"
        elif tox_opts["proxy_type"] != ToxCore.TOX_PROXY_TYPE_NONE:
            raise NotImplementedError("Unknown proxy_type: {0}".format(tox_opts["proxy_type"]))

        return options


    @staticmethod
    def loadOptions(filename, options = None):
        """
        Чтение секции echobot INI файла echobot.cfg

        Аргументы:
            filename (str)  -- Имя INI файла
            options  (dict) -- Базовая конфигурация

        Результат (dict):
            Конфигурация приложения на основе файла конфигурации
        """
        if options is None:
            options = EchoBotOptions.defaultOptions()

        options = options.copy()

        parser = EchoBotConfigParser.ConfigParser()

        parser.read(filename)

        for section in parser.sections():
            name = section.lower()
            if name == "echobot":
                for option in parser.options(section):
                    options[option.lower()] = parser.get(section, option).strip()

        return options


class EchoBot(ToxCore):
    """
    Бот
    """
    def __init__(self, options):
        """
        Аргументы:
            options (EchoBotOptions) -- Опции приложения
        """
        self.options = options

        tox_opts = {
            "ipv6_enabled" : self.options.ipv6_enabled,
            "udp_enabled"  : self.options.udp_enabled,
            "proxy_type"   : self.options.proxy_type,
            "proxy_host"   : self.options.proxy_host,
            "proxy_port"   : self.options.proxy_port,
            "start_port"   : self.options.start_port,
            "end_port"     : self.options.end_port,
            "tcp_port"     : self.options.tcp_port
        }

        if os.path.isfile(self.options.save_file):
            self.debug("Load data from file: {0}".format(self.options.save_file))
            with open(self.options.save_file, "rb") as f:
                tox_opts["savedata_type"] = ToxCore.TOX_SAVEDATA_TYPE_TOX_SAVE
                tox_opts["savedata_data"] = f.read()

        super(EchoBot, self).__init__(tox_opts)

        self.debug("Set self name: {0}".format(self.options.name))
        self.tox_self_set_name(self.options.name)

        self.debug("Set self status: {0}".format(self.options.status_message))
        self.tox_self_set_status_message(self.options.status_message)

        self.debug("Get self ToxID: {0}".format(self.tox_self_get_address()))


    def debug(self, message):
        """
        Вывод отладочной информации

        Аргументы:
            message (str) -- Сообщение для вывода
        """
        if self.options.debug:
            sys.stderr.write("[{0}] {1}\n".format(time.strftime("%Y-%m-%d %H:%M:%S"), message))


    def verbose(self, message):
        """
        Вывод расширенной информации

        Аргументы:
            message (str) -- Сообщение для вывода
        """
        if self.options.verbose:
            sys.stderr.write("[{0}] {1}\n".format(time.strftime("%Y-%m-%d %H:%M:%S"), message))


    def run(self):
        """
        Рабочий цикл
        """
        self.debug("Connecting to: {0} {1} {2}".format(self.options.bootstrap_host, self.options.bootstrap_port, self.options.bootstrap_key))
        self.tox_bootstrap(self.options.bootstrap_host, self.options.bootstrap_port, self.options.bootstrap_key)
        self.debug("Connected to: {0} {1} {2}".format(self.options.bootstrap_host, self.options.bootstrap_port, self.options.bootstrap_key))

        checked       = False
        savetime      = 0
        save_interval = self.options.save_interval * 1000

        while True:
            status = self.tox_self_get_connection_status()

            if not checked and status != ToxCore.TOX_CONNECTION_NONE:
                checked = True

            if checked and status == ToxCore.TOX_CONNECTION_NONE:
                self.debug("Connecting to: {0} {1} {2}".format(self.options.bootstrap_host, self.options.bootstrap_port, self.options.bootstrap_key))
                self.tox_bootstrap(self.options.bootstrap_host, self.options.bootstrap_port, self.options.bootstrap_key)
                self.debug("Connected to: {0} {1} {2}".format(self.options.bootstrap_host, self.options.bootstrap_port, self.options.bootstrap_key))
                checked = False

            self.tox_iterate()

            interval = self.tox_iteration_interval()

            time.sleep(float(interval) / 1000.0)

            savetime += interval
            if savetime > save_interval:
                self.save_file()
                savetime = 0


    def save_file(self):
        """
        Сохранение данных
        """
        self.debug("Save data to file: {0}".format(self.options.save_tmp_file))

        with open(self.options.save_tmp_file, "wb") as f:
            f.write(self.tox_get_savedata())

        self.debug("Move data to file: {0}".format(self.options.save_file))
        os.rename(self.options.save_tmp_file, self.options.save_file)


    def send_avatar(self, friend_number):
        """
        Отправка аватара

        Агрументы:
            friend_number (int) -- Номер друга
        """
        if len(self.options.avatar) == 0 or not os.path.isfile(self.options.avatar):
            return

        friend_name = self.tox_friend_get_name(friend_number)
        self.verbose("Send avatar to {0}/{1}".format(friend_name, friend_number))

        self.tox_sendfile(friend_number, ToxCore.TOX_FILE_KIND_AVATAR, self.options.avatar, "", 60)


    def send_file(self, friend_number, path, name = None):
        """
        Отправка файла

        Аргументы:
            friend_number (int) -- Номер друга
            path          (str) -- Путь к файлу
            name          (str) -- Имя файла (опционально, если не совпадает с именем из path)
        """
        if not os.path.isfile(path):
            return

        friend_name = self.tox_friend_get_name(friend_number)

        if name:
            self.verbose("Send file {0} as {1} to {2}/{3}".format(path, name, friend_name, friend_number))
        else:
            self.verbose("Send file {0} to {1}/{2}".format(path, friend_name, friend_number))

        if not name:
            name = os.path.basename(path)

        self.tox_sendfile(friend_number, ToxCore.TOX_FILE_KIND_DATA, path, name, 60)


    def tox_self_connection_status_cb(self, connection_status):
        """
        Изменение состояния соединения

        Аргументы:
            connection_status (int) -- Статус
        """
        if connection_status == ToxCore.TOX_CONNECTION_NONE:
            self.debug("Disconnected from DHT")
        elif connection_status == ToxCore.TOX_CONNECTION_TCP:
            self.debug("Connected to DHT via TCP")
        elif connection_status == ToxCore.TOX_CONNECTION_UDP:
            self.debug("Connected to DHT via UDP")
        else:
            raise NotImplementedError("Unknown connection_status: {0}".format(connection_status))


    def tox_friend_request_cb(self, public_key, message):
        """
        Запрос на добавление в друзья

        Аргументы:
            public_key (str) -- Публичный ключ друга
            message    (str) -- Сообщение запроса для добавления в друзья
        """
        self.verbose("Friend request from {0}: {1}".format(public_key, message))
        self.tox_friend_add_norequest(public_key)
        self.verbose("Friend request from {0}: accepted".format(public_key))


    def tox_friend_connection_status_cb(self, friend_number, connection_status):
        """
        Изменение состояния соединения друга

        Агрументы:
            friend_number     (int) -- Номер друга
            connection_status (int) -- Статус соединения друга (см. enum TOX_CONNECTION)
        """
        friend_name = self.tox_friend_get_name(friend_number)

        if connection_status == ToxCore.TOX_CONNECTION_NONE:
            self.verbose("Friend {0}/{1} is offline".format(friend_name, friend_number))
        elif connection_status == ToxCore.TOX_CONNECTION_TCP:
            self.verbose("Friend {0}/{1} connected via TCP".format(friend_name, friend_number))
        elif connection_status == ToxCore.TOX_CONNECTION_UDP:
            self.verbose("Friend {0}/{1} connected via UDP".format(friend_name, friend_number))
        else:
            raise NotImplementedError("Unknown connection_status: {0}".format(connection_status))

        if connection_status == ToxCore.TOX_CONNECTION_TCP or connection_status == ToxCore.TOX_CONNECTION_UDP:
            self.send_avatar(friend_number)


    def tox_friend_name_cb(self, friend_number, name):
        """
        Смена имени друга

        Аргументы:
            friend_number (int) -- Номер друга
            name          (str) -- Новое имя
        """
        self.verbose("Friend name change {0}/{1}".format(name, friend_number))


    def tox_friend_status_message_cb(self, friend_number, message):
        """
        Смена сообщения статуса

        Аргументы:
            friend_number (int) -- Номер друга
            message       (str) -- Сообщение статуса
        """
        friend_name = self.tox_friend_get_name(friend_number)

        self.verbose("Friend status message change {0}/{1}: {2}".format(friend_name, friend_number, message))


    def tox_friend_status_cb(self, friend_number, status):
        """
        Смена статуса

        Аргументы:
            friend_number (int) -- Номер друга
            status        (int) -- Статус (см. TOX_USER_STATUS)
        """
        friend_name = self.tox_friend_get_name(friend_number)

        if status ==ToxCore.TOX_USER_STATUS_NONE:
            self.verbose("Friend {0}/{1} is online now".format(friend_name, friend_number))
        elif status == ToxCore.TOX_USER_STATUS_AWAY:
            self.verbose("Friend {0}/{1} is away now".format(friend_name, friend_number))
        elif status == ToxCore.TOX_USER_STATUS_BUSY:
            self.verbose("Friend {0}/{1} is busy now".format(friend_name, friend_number))
        else:
            raise NotImplementedError("Unknown status: {0}".format(status))


    def tox_friend_message_cb(self, friend_number, message):
        """
        Сообщение от друга

        Аргументы:
            friend_number (int) -- Номер друга
            message       (str) -- Сообщение
        """
        friend_name = self.tox_friend_get_name(friend_number)

        self.verbose("Message from {0}/{1}: {2}".format(friend_name, friend_number, message))
        message_id = self.tox_friend_send_message(friend_number, ToxCore.TOX_MESSAGE_TYPE_NORMAL, message)
        self.verbose("Message {0} to {1}/{2}: {3}".format(message_id, friend_name, friend_number, message))


    def tox_friend_read_receipt_cb(self, friend_number, message_id):
        """
        Квитанция о доставке сообщения

        Аргументы:
            friend_number (int) -- Номер друга
            message_id    (int) -- ID сообщения
        """
        friend_name = self.tox_friend_get_name(friend_number)

        self.verbose("Message receipt {0} from {1}/{2}".format(message_id, friend_name, friend_number))


    def can_accept_file(self, friend_number, file_number, kind, file_size, filename):
        """
        Проверка, что файл можно принимать

        Аргументы:
            friend_number (int) -- Номер друга
            file_number   (int) -- Номер файла (случайный номер в рамках передачи)
            kind          (int) -- Значение файла (см. TOX_FILE_KIND)
            file_size     (int) -- Размер файла
            filename      (str) -- Имя файла

        Результат (bool):
            Флаг разрешения на принятие файла
        """
        # поток?
        if file_size <= 0:
            return False

        if kind == ToxCore.TOX_FILE_KIND_DATA:
            return (
                self.options.accept_files and
                (self.options.max_file_size == 0 or file_size <= self.options.max_file_size) and
                (os.path.isdir(self.options.files_path)))

        elif kind == ToxCore.TOX_FILE_KIND_AVATAR:
            return (
                self.options.accept_avatars and
                (self.options.max_avatar_size == 0 or file_size <= self.options.max_avatar_size) and \
                (os.path.isdir(self.options.avatars_path)))

        raise NotImplementedError("Unknown kind: {0}".format(kind))


    def tox_file_recv_cb(self, friend_number, file_number, kind, file_size, filename):
        """
        Получение файла
        (см. tox_file_recv_cb)

        Аргументы:
            friend_number (int) -- Номер друга
            file_number   (int) -- Номер файла (случайный номер в рамках передачи)
            kind          (int) -- Значение файла (см. TOX_FILE_KIND)
            file_size     (int) -- Размер файла
            filename      (str) -- Имя файла
        """
        friend_name = self.tox_friend_get_name(friend_number)

        if kind == ToxCore.TOX_FILE_KIND_DATA:
            file_id = self.tox_file_get_file_id(friend_number, file_number)
            self.verbose("File from {0}/{1}: number = {2}, size = {3}, id = {4}, name = {5}".format(friend_name, friend_number, file_number, file_size, file_id, filename))
        elif kind == ToxCore.TOX_FILE_KIND_AVATAR:
            filename = ""
            if file_size != 0:
                file_id = self.tox_file_get_file_id(friend_number, file_number)
                self.verbose("Avatar from {0}/{1}: number = {2}, size = {3}, id = {4}".format(friend_name, friend_number, file_number, file_size, file_id))
            else:
                self.verbose("No Avatar from {0}/{1}: number = {2}".format(friend_name, friend_number, file_number))
        else:
            raise NotImplementedError("Unknown kind: {0}".format(kind))

        if self.can_accept_file(friend_number, file_number, kind, file_size, filename):
            if kind == ToxCore.TOX_FILE_KIND_DATA:
                path = self.options.files_path + "/" + file_id
            elif kind == ToxCore.TOX_FILE_KIND_AVATAR:
                path = self.options.avatars_path + "/" + file_id

            self.tox_recvfile(friend_number, file_number, file_size, path, filename, 60)
        else:
            self.tox_file_control(friend_number, file_number, ToxCore.TOX_FILE_CONTROL_CANCEL)


    def tox_file_recv_control_cb(self, friend_number, file_number, control):
        """
        Контроль получения файла
        (см. tox_file_recv_control_cb)

        Аргументы:
            friend_number (int) -- Номер друга
            file_number   (int) -- Номер файла (случайный номер в рамках передачи)
            control       (int) -- Полученная команда контроля (см. TOX_FILE_CONTROL)
        """
        friend_name = self.tox_friend_get_name(friend_number)

        if control == ToxCore.TOX_FILE_CONTROL_RESUME:
            self.verbose("File resumed from {0}/{1}: number = {2}".format(friend_name, friend_number, file_number))
        elif control == ToxCore.TOX_FILE_CONTROL_PAUSE:
            self.verbose("File paused from {0}/{1}: number = {2}".format(friend_name, friend_number, file_number))
        elif control == ToxCore.TOX_FILE_CONTROL_CANCEL:
            self.verbose("File canceled from {0}/{1}: number = {2}".format(friend_name, friend_number, file_number))
        else:
            raise NotImplementedError("Unknown control: {0}".format(control))


    def tox_recvfile_cb(self, friend_number, file_number, path, filename, status):
        """
        Контроль выполнения tox_recvfile

        Аргументы:
            friend_number (int) -- Номер друга
            file_number   (int) -- Номер файла (случайный номер в рамках передачи)
            status        (int) -- Статус получения файла
        """
        friend_name = self.tox_friend_get_name(friend_number)

        if status == ToxCore.TOX_RECVFILE_COMPLETED:
            self.verbose("recvfile completed to {0}/{1}: number = {2}".format(friend_name, friend_number, file_number))

            file_id = self.tox_file_get_file_id(friend_number, file_number)
            path = self.options.files_path + "/" + file_id
            if os.path.isfile(path):
                self.tox_sendfile(friend_number, ToxCore.TOX_FILE_KIND_DATA, path, filename, 60)

        elif status == ToxCore.TOX_RECVFILE_TIMEOUT:
            self.verbose("recvfile timeout to {0}/{1}: number = {2}".format(friend_name, friend_number, file_number))
        elif status == ToxCore.TOX_RECVFILE_ERROR:
            self.verbose("recvfile error to {0}/{1}: number = {2}".format(friend_name, friend_number, file_number))
        else:
            raise NotImplementedError("Unknown status: {0}".format(status))


if __name__ == "__main__":
    regexp  = re.compile("--config=(.*)")
    cfgfile = [match.group(1) for arg in sys.argv for match in [regexp.search(arg)] if match]

    if len(cfgfile) == 0:
        cfgfile = "echobot.cfg"
    else:
        cfgfile = cfgfile[0]

    options = EchoBotOptions(EchoBotOptions.loadOptions(cfgfile))

    bot = EchoBot(options)

    try:
        bot.run()
    except KeyboardInterrupt:
        bot.save_file()
